@@ -38,10 +38,10 @@ def list_variables(Print_Variable_List: bool=False):
                 ## EXCLUDE VARIABLES THAT START WITH '__'
                 if not var_name.startswith('__'):
 
+                        ## GET VALUES
                         try:
-
                                 ## READ VARIABLE VALUE
-                                var_value = eval(var_name)
+                                var_value = __main__.__dict__[var_name]
 
                                 ## EXCLUDE MODULES AND FUNCTIONS
                                 if not str(type(var_value)) == "<class 'module'>" and not str(type(var_value)) == "<class 'function'>":
@@ -49,20 +49,27 @@ def list_variables(Print_Variable_List: bool=False):
                                         ## READ VARIABLE TYPE
                                         var_type = strgs.clean_text(strgs.filter_text(str(type(var_value)), ['class ']))
 
-                                        ## PRINT: VARIABLE NAME, TYPE AND VALUE
-                                        if Print_Variable_List:
-                                                print(f"Variable | {var_name}\n"
-                                                        f"Type     | {var_type}\n"
-                                                        f"Value    | {var_value}\n")
+                                        ## CHECKS ONLY FOR...
+                                        # INT, FLOAT, STR, LIST, TUPLE, DICT, SET, OBJECT, COMPLEX
+                                        if var_type == 'int' or var_type == 'float' or var_type == 'str' or var_type == 'list' or var_type == 'tuple' or var_type == 'dict' or var_type == 'set' or var_type == 'object' or var_type == 'complex':
 
-                                        ## APPEND OUR VARIABLE LIST
-                                        n4s_var_list.append(var_name)
+                                                ## PRINT: VARIABLE NAME, TYPE AND VALUE
+                                                if Print_Variable_List:
+                                                        print(f"Variable | {var_name}\n"
+                                                                f"Type     | {var_type}\n"
+                                                                f"Value    | {var_value}\n")
+
+                                                ## APPEND OUR VARIABLE LIST
+                                                n4s_var_list.append(var_name)
 
                                 ## EXCLUDE VARIABLES FROM THIS FUNCTION
                                 if var_name == 'Print_Variable_List' or var_name == 'n4s_var_list':
                                         pass
+                        
+                        ## ERROR
                         except Exception:
-                                pass
+                                return print("\nn4s.term.list_variables():\n"
+                                        f"Unable to process request\n")
         
         ## PRINT: VARIABLE LIST TO TERMINAL
         if Print_Variable_List:
